==> eb123-0.1.tar.gz: eb123-0.1/src/selection.c <==

#include "defs.h"
#include "ebook.h"
#include "headword.h"
#include "history.h"
#include "jcode.h"
#include "popupwnd.h"
#include "selection.h"

gint selection_copy(gpointer data)
{
    GtkClipboard* clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
    gchar *str = gtk_clipboard_wait_for_text(clipboard);
    static gchar *sel = NULL, *sel_prev = NULL;
    if(!str)
    {
        g_free(sel_prev);
        sel_prev = NULL;
        return TRUE;
    }
    g_strstrip(str);
    str = g_strdelimit(str, G_STR_DELIMITERS, '\n');
    gint len = strlen(str);
    if((len > selection.maxchar) || g_strrstr(str, "\n"))
    {
        g_free(sel_prev);
        sel_prev = NULL;
        g_free(str);
        return TRUE;
    }
    if(!sel)
    {
        sel = str;
        return TRUE;
    }
    if(strncmp(str, sel, selection.maxchar))
    {
        g_free(sel);
        sel = str;
        return TRUE;
    }
    g_free(str);
    if(sel_prev ? !strncmp(sel_prev, sel, selection.maxchar) : FALSE)
        return TRUE;
    popupwnd_search(sel);
    sel_prev = sel;
    sel = NULL;
    return TRUE;
}

void selection_lookup_start()
{
    if(selection.timeout_id != 0)
        g_source_remove(selection.timeout_id);
    selection.timeout_id = g_timeout_add(selection.interval, selection_copy, NULL);
    selection.lookup = TRUE;
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(popupwnd.scan), TRUE);
}

void selection_lookup_stop()
{
    if(selection.timeout_id != 0)
        g_source_remove(selection.timeout_id);
    selection.timeout_id = 0;
    selection.lookup = FALSE;
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(popupwnd.scan), FALSE);
    popupwnd_close(NULL, NULL);
}

void selection_toggled(GtkToggleButton *togglebutton, gpointer data)
{
    if(gtk_toggle_button_get_active(togglebutton))
        selection_lookup_start();
    else
        selection_lookup_stop();
}

==> eb123-0.2.tar.gz: eb123-0.2/src/selection.c <==

#include "defs.h"
#include "ebook.h"
#include "headword.h"
#include "history.h"
#include "jcode.h"
#include "popupwnd.h"
#include "selection.h"

gint selection_copy(gpointer data)
{
    GtkClipboard* clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
    gchar *str = gtk_clipboard_wait_for_text(clipboard);
    static gchar *sel = NULL, *sel_prev = NULL;
    if(!str || selection.lookup_suspended)
    {
        if(sel_prev)
            g_free(sel_prev);
        sel_prev = NULL;
        return 1;
    }
    g_strstrip(str);
    str = g_strdelimit(str, G_STR_DELIMITERS, '\n');
    gint len = strlen(str);
    if((len > selection.maxchar) || g_strrstr(str, "\n"))
    {
        g_free(sel_prev);
        sel_prev = NULL;
        g_free(str);
        return 1;
    }
    if(!sel)
    {
        sel = str;
        return 1;
    }
    if(strncmp(str, sel, selection.maxchar))
    {
        g_free(sel);
        sel = str;
        return 1;
    }
    g_free(str);
    if(sel_prev ? !strncmp(sel_prev, sel, selection.maxchar) : FALSE)
        return 1;
    popupwnd_search(sel);
    sel_prev = sel;
    sel = NULL;
    return 1;
}

void selection_lookup_start()
{
    selection.lookup_started = TRUE;
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(popupwnd.scan), TRUE);
    selection_lookup_resume();
}

void selection_lookup_stop()
{
    selection.lookup_started = FALSE;
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(popupwnd.scan), FALSE);
    selection_lookup_suspend();
}

void selection_lookup_suspend()
{
    if(selection.timeout_id != 0)
        g_source_remove(selection.timeout_id);
    selection.timeout_id = 0;
    popupwnd_close(NULL, NULL);
    selection.lookup_suspended = TRUE;
}

void selection_lookup_resume()
{
    if(selection.lookup_started)
    {
        if(selection.timeout_id != 0)
            g_source_remove(selection.timeout_id);
        selection.timeout_id = g_timeout_add(selection.interval, selection_copy, NULL);
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(popupwnd.scan), TRUE);
    }
    selection.lookup_suspended = FALSE;
}

void selection_toggled(GtkToggleButton *togglebutton, gpointer data)
{
    if(gtk_toggle_button_get_active(togglebutton))
        selection_lookup_start();
    else
        selection_lookup_stop();
}
